import { expect } from 'chai';
import { configure, shallow } from 'enzyme';
import Adapter from 'enzyme-adapter-react-16';
import React from 'react';
import App from '../src/app';
import sinon from 'sinon';

configure({ adapter: new Adapter() });

describe('App Component Tests', function() {
  let sandbox;
  let clock;
  
  beforeEach(() => {
    sandbox = sinon.createSandbox();
    clock = sandbox.useFakeTimers();
  });
  
  afterEach(() => sandbox.restore());
  
  it('canary test', () => {
    expect(true).to.be.true;
  });
  
  it('App renders div', () => {
    expect(shallow(<App/>).html().includes('<div>')).to.be.true;
  });
  
  it("App renders today's date", () => {
    const today = new Date().toDateString();
    expect(shallow(<App/>).text().includes(today)).to.be.true;
  });                                                         
  
  it('App renders current time', () => {
    const faketime = `faketime${Math.random()}`;
    sandbox.stub(Date.prototype, 'toLocaleTimeString')
      .returns(faketime);                       
    const today = new Date().toDateString();
    
    expect(shallow(<App/>).text().includes(`${today} faketime`)).to.be.true;
  });
  
  it('componentDidMount calls setInterval which calls setState', () => {
    const app = new App();
    const setStateStub = sandbox.stub(app, 'setState')
      .withArgs({});
    
    app.componentDidMount();

    expect(setStateStub.called).to.be.false;
    clock.tick(1000);
    expect(setStateStub.called).to.be.true;
  });
  
  it('componentDidMount stored intervalID', () => {
    const app = new App();

    app.componentDidMount();     
      
    expect(app.intervalID.id).to.be.eql(clock.timers[app.intervalID.id].id);
  });
  
  it('stopTimer calls clearInterval with stored intervalID', () => {
    const app = new App();
    app.componentDidMount();     
    
    app.stopTimer();
    
    expect(clock.timers).to.be.eql({});
  });
  
  it('App renders stop button', () => {
    expect(shallow(<App/>).html().includes('<button>Stop</button>')).to.be.true;    
  });
  
  it('stop button click calls stopTimer', () => {
    const stopTimerStub = sandbox.stub(App.prototype, 'stopTimer');
    
    const wrapper = shallow(<App/>);
    const button = wrapper.find('button');
    
    button.simulate('click');                                        
    
    expect(stopTimerStub.called).to.be.true;
  });
});